/******************************************************************************/
/* AGENT2B                                                                    */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome:                                                                      */
/*                                                                            */
/* Descrio:                                                                 */
/*    Faz o fechamento de uma AF                                              */
/*                                                                            */
/* Criao:                                                                   */
/*   07/07/2003 - Adriano de Arajo                                           */
/*                                                                            */
/******************************************************************************/
/* @C:\Web_V2\Srv\Src\Sup\Fechamento_AF.sql                                   */
/******************************************************************************/


CREATE OR REPLACE PROCEDURE FECHAMENTO_AF
                     (P_CDFILDEST  AUTOFORNE.CDFILDEST%TYPE,
                      P_CDFILLANC  AUTOFORNE.CDFILLANC%TYPE,
                      P_NRAUTOFORN AUTOFORNE.NRAUTOFORN%TYPE)
IS
  V_CDFORNECED  FORNECEDOR.CDFORNECED%TYPE;
  V_DTPROGENTAF ITEMAUTF.DTPROGENTAF%TYPE;
  V_CDEMPRESA   EMPRESA.CDEMPRESA%TYPE;
  V_VRIMPOSTO   FLOAT;
  V_VRENCAFPOS  FLOAT;
  V_VRENCAFNEG  FLOAT;
  V_CDOPERADOR  OPERADOR.CDOPERADOR%TYPE;
  V_CDINSCESTA  EMPRESA.CDINSCESTA%TYPE;
  V_VRPRODUTO   FLOAT;
  V_CDPRODESTO  PRODUTO.CDPRODESTO%TYPE;
  V_VRDESCONTO  FLOAT;
  V_CDTIPCOB    FORNECEDOR.CDTIPCOB%TYPE;

BEGIN
   /* Dados da AF */
   SELECT
     MAX(AUTOFORNE.CDFORNECED),
     NVL(MAX(AUTOFORNE.CDEMPRESA), MAX(FILIAL.CDEMPRESA)),
     MAX(AUTOFORNE.CDOPERADOR) INTO V_CDFORNECED,
                                    V_CDEMPRESA,
                                    V_CDOPERADOR
   FROM
     AUTOFORNE,
     FILIAL
   WHERE
     FILIAL.CDFILIAL = AUTOFORNE.CDFILDEST AND
     CDFILDEST       = P_CDFILDEST         AND
     CDFILLANC       = P_CDFILLANC         AND
     NRAUTOFORN      = P_NRAUTOFORN;

   /* Gerao das Parcelas */
   UPDATE ITEMAUTF
   SET NRPARCITMAF = (SELECT
                        LPAD(TO_CHAR(COUNT(DISTINCT ITEM.DTPROGENTAF)),2,'0')
                      FROM
                        ITEMAUTF ITEM
                      WHERE
                        ITEM.CDFILDEST   = P_CDFILDEST  AND
                        ITEM.CDFILLANC   = P_CDFILLANC  AND
                        ITEM.NRAUTOFORN  = P_NRAUTOFORN AND
                        ITEM.DTPROGENTAF <= ITEMAUTF.DTPROGENTAF)
   WHERE
      CDFILDEST  = P_CDFILDEST AND
      CDFILLANC  = P_CDFILLANC AND
      NRAUTOFORN = P_NRAUTOFORN;

   V_CDINSCESTA := NULL;
   SELECT
     MIN(CDINSCESTA) INTO V_CDINSCESTA
   FROM
     FILIAL
   WHERE
     CDFILIAL  = P_CDFILDEST AND
     CDEMPRESA = V_CDEMPRESA;

   IF V_CDINSCESTA IS NULL THEN
      SELECT
        MIN(CDINSCESTA) INTO V_CDINSCESTA
      FROM
        EMPRCNTRFIL
      WHERE
        CDEMPRESA = V_CDEMPRESA;
   END IF;
   IF V_CDINSCESTA IS NULL THEN
      SELECT
        MIN(CDINSCESTA) INTO V_CDINSCESTA
      FROM
        INSCRESTAD
      WHERE
        CDEMPRESA = V_CDEMPRESA;
   END IF;

   SELECT
     MIN(DTPROGENTAF) INTO V_DTPROGENTAF
   FROM
     ITEMAUTF
   WHERE
     CDFILDEST  = P_CDFILDEST AND
     CDFILLANC  = P_CDFILLANC AND
     NRAUTOFORN = P_NRAUTOFORN;

   IMPOSTO_ITEMAF(V_CDEMPRESA,
                  V_CDINSCESTA,
                  V_CDFORNECED,
                  P_CDFILDEST,
                  P_CDFILLANC,
                  P_NRAUTOFORN,
                  V_DTPROGENTAF);

   /* Atualizao do Valor Total da AF */
   SELECT
      SUM(ITEMAUTF.QTPROGENTAF * ITEMAUTF.VRPREUNIAF),
      SUM((ITEMAUTF.QTPROGENTAF * ITEMAUTF.VRPREUNIAF * NVL(ITEMAUTF.VRPERDESAF, 0)) / 100)
      INTO V_VRPRODUTO, V_VRDESCONTO
   FROM
      ITEMAUTF
   WHERE
      CDFILDEST  = P_CDFILDEST AND
      CDFILLANC  = P_CDFILLANC AND
      NRAUTOFORN = P_NRAUTOFORN;

   SELECT
     NVL(SUM(IMPITMAF.VRADIMPAF), 0) INTO V_VRIMPOSTO
   FROM
     IMPITMAF,
     ITEMAUTF
   WHERE
      IMPITMAF.CDFILDEST   = P_CDFILDEST          AND
      IMPITMAF.CDFILLANC   = P_CDFILLANC          AND
      IMPITMAF.NRAUTOFORN  = P_NRAUTOFORN         AND
      IMPITMAF.CDFILDEST   = ITEMAUTF.CDFILDEST   AND
      IMPITMAF.CDFILLANC   = ITEMAUTF.CDFILLANC   AND
      IMPITMAF.NRAUTOFORN  = ITEMAUTF.NRAUTOFORN  AND
      IMPITMAF.CDPRODUTO   = ITEMAUTF.CDPRODUTO   AND
      IMPITMAF.CDMARCA     = ITEMAUTF.CDMARCA     AND
      IMPITMAF.DTPROGENTAF = ITEMAUTF.DTPROGENTAF;

   /* BUSCA A PARTE POSITIVA DO ENCARGO */
   SELECT
     NVL(SUM(ENCARGOAF.VRENCARGOAF), 0) INTO V_VRENCAFPOS
   FROM
     ENCARGOAF,
     ENCARGO
   WHERE
     ENCARGO.IDENCARGO    = 'A'          AND
     ENCARGOAF.CDFILDEST  = P_CDFILDEST  AND
     ENCARGOAF.CDFILLANC  = P_CDFILLANC  AND
     ENCARGOAF.NRAUTOFORN = P_NRAUTOFORN AND
     ENCARGO.CDENCARGO    = ENCARGOAF.CDENCARGO;

   /* BUSCA A PARTE NEGATIVA DO ENCARGO */
   SELECT
     NVL(SUM(ENCARGOAF.VRENCARGOAF), 0) INTO V_VRENCAFNEG
   FROM
     ENCARGOAF,
     ENCARGO
   WHERE
     ENCARGO.IDENCARGO    = 'D'          AND
     ENCARGOAF.CDFILDEST  = P_CDFILDEST  AND
     ENCARGOAF.CDFILLANC  = P_CDFILLANC  AND
     ENCARGOAF.NRAUTOFORN = P_NRAUTOFORN AND
     ENCARGO.CDENCARGO    = ENCARGOAF.CDENCARGO;

   SELECT MAX(CDTIPCOB)
     INTO V_CDTIPCOB
     FROM FORNECEDOR
    WHERE CDFORNECED = V_CDFORNECED;

   UPDATE AUTOFORNE
   SET VRTOTALAF = V_VRPRODUTO - V_VRDESCONTO + V_VRIMPOSTO + V_VRENCAFPOS - V_VRENCAFNEG,
       CDTIPCOB  = V_CDTIPCOB
   WHERE
      CDFILDEST  = P_CDFILDEST AND
      CDFILLANC  = P_CDFILLANC AND
      NRAUTOFORN = P_NRAUTOFORN;

   --COMMIT;

   /* Gravao do Custo da ltima AF*/
   V_CDPRODESTO := '';

   FOR C_PrecMedio IN
     (SELECT
        ITEMAUTF.CDFILDEST,
        AUTOFORNE.DTAUTOFORN,
        PRODUTO.CDPRODESTO,
        TIPOCUSTO.CDTIPOCUSTO,
        TIPOCUSTO.IDMOEDCUSTO,
        AVG(ITEMAUTF.VRPREUNIAF/PRODUTO.VRFATOCONV) AS VRMEDIO
     FROM
        TIPOCUSTO,
        PARAMGERAL,
        PRODUTO,
        AUTOFORNE,
        ITEMAUTF
     WHERE
        ITEMAUTF.CDFILLANC  = P_CDFILLANC  AND
        ITEMAUTF.CDFILDEST  = P_CDFILDEST  AND
        ITEMAUTF.NRAUTOFORN = P_NRAUTOFORN AND
        ITEMAUTF.CDFILLANC  = AUTOFORNE.CDFILLANC  AND
        ITEMAUTF.CDFILDEST  = AUTOFORNE.CDFILDEST  AND
        ITEMAUTF.NRAUTOFORN = AUTOFORNE.NRAUTOFORN AND

        TIPOCUSTO.CDTIPOCUSTO IN ('07', '32', '33') AND

        PRODUTO.CDPRODUTO   = ITEMAUTF.CDPRODUTO
     GROUP BY
        ITEMAUTF.CDFILDEST,
        AUTOFORNE.DTAUTOFORN,
        PRODUTO.CDPRODESTO,
        TIPOCUSTO.CDTIPOCUSTO,
        TIPOCUSTO.IDMOEDCUSTO
     ORDER BY
        ITEMAUTF.CDFILDEST,
        AUTOFORNE.DTAUTOFORN,
        PRODUTO.CDPRODESTO)
   LOOP
      IF V_CDPRODESTO <> C_PrecMedio.CDPRODESTO THEN
         UPDATE PRODFILI
         SET DTULTAUTFOR  = C_PrecMedio.DTAUTOFORN,
             VRPRECUAFOFI = C_PrecMedio.VRMEDIO,
             VRPRECUAFOM1 = (SELECT NVL(MAX(MOVINDICE.VRINDICE), 0) * NVL(C_PrecMedio.VRMEDIO, 0)
                             FROM PARAMGERAL, MOVINDICE
                             WHERE PARAMGERAL.CDINDICEMF = MOVINDICE.CDINDICE AND
                                   MOVINDICE.DTREFIND    = C_PrecMedio.DTAUTOFORN AND
                                   MOVINDICE.CDTPINDICE  = '1' AND
                                   MOVINDICE.IDPERVALOR  = '2'),
             VRPRECUAFOM2 = (SELECT NVL(MAX(MOVINDICE.VRINDICE), 0) * NVL(C_PrecMedio.VRMEDIO, 0)
                             FROM PARAMGERAL, MOVINDICE
                             WHERE PARAMGERAL.CDINDICEMF2 = MOVINDICE.CDINDICE AND
                                   MOVINDICE.DTREFIND     = C_PrecMedio.DTAUTOFORN AND
                                   MOVINDICE.CDTPINDICE   = '1' AND
                                   MOVINDICE.IDPERVALOR   = '2')
         WHERE
            CDFILIAL  = P_CDFILDEST AND
            CDPRODUTO = C_PrecMedio.CDPRODESTO;
      
        --COMMIT;
      END IF;
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('FECHAMENTO_AF','4.07.006'); END;
/

